@@ -1,12 +1,8 @@
 /* ============================================================
- *
- * This file is part of the KDE project
- *
- * Date        : 2009-01-21
- * Description : Sane interface for KDE
  *
  * SPDX-FileCopyrightText: 2009 Kare Sars <kare dot sars at iki dot fi>
  * SPDX-FileCopyrightText: 2014 Gregor Mitsch : port to KDE5 frameworks
+ * SPDX-FileCopyrightText: 2021 Alexander Stippich <a.stippich@gmx.net>
  *
  * SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
  *
@@ -15,36 +11,28 @@
 #ifndef KSANE_WIDGET_PRIVATE_H
 #define KSANE_WIDGET_PRIVATE_H
 
-// Sane includes
-extern "C"
-{
-#include <sane/saneopts.h>
-#include <sane/sane.h>
-}
-
 #include <QWidget>
 #include <QCheckBox>
 #include <QTimer>
-#include <QTime>
 #include <QProgressBar>
 #include <QTabWidget>
 #include <QPushButton>
 #include <QVector>
 #include <QSplitter>
 #include <QToolButton>
+#include <QSet>
 
+#include "ksanecore.h"
 #include "ksanewidget.h"
-#include "ksaneoption.h"
+#include "ksaneoptionwidget.h"
 #include "ksaneviewer.h"
 #include "labeledcombo.h"
 #include "labeledfslider.h"
+#include "labeledentry.h"
+#include "ksanebutton.h"
 #include "labeledgamma.h"
 #include "labeledcheckbox.h"
 #include "splittercollapser.h"
-#include "ksanescanthread.h"
-#include "ksanepreviewthread.h"
-#include "ksanefinddevicesthread.h"
-#include "ksaneauth.h"
 
 #define IMG_DATA_R_SIZE 100000
 
@@ -60,11 +48,9 @@ public:
     void clearDeviceOptions();
     void createOptInterface();
     void updatePreviewSize();
-    void setDefaultValues();
     void setBusy(bool busy);
-    KSaneOption *getOption(const QString &name);
-    KSaneWidget::ImageFormat getImgFormat(SANE_Parameters &params);
-    int getBytesPerLines(SANE_Parameters &params);
+    KSaneOptionWidget *createOptionWidget(QWidget *parent, KSaneOption *option);
+    KSaneWidget::ImageFormat getImgFormat(const QImage &image);
 
     float ratioToScanAreaX(float ratio);
     float ratioToScanAreaY(float ratio);
@@ -79,36 +65,39 @@ public:
     bool scanSourceADF();
 
 public Q_SLOTS:
-    void devListUpdated();
-    void signalDevListUpdate();
     void startFinalScan();
     void startPreviewScan();
-    void previewScanDone();
-    void oneFinalScanDone();
-    void updateProgress();
-
-private Q_SLOTS:
-    void scheduleValReload();
-    void optReload();
-    void valReload();
+    void scanDone(KSaneCore::KSaneScanStatus status, const QString &strStatus);
+    void previewScanDone(KSaneCore::KSaneScanStatus status, const QString &strStatus);
+    void oneFinalScanDone(KSaneCore::KSaneScanStatus status, const QString &strStatus);
+    void updateProgress(int progress);
     void handleSelection(float tl_x, float tl_y, float br_x, float br_y);
-    void setTLX(float x);
-    void setTLY(float y);
-    void setBRX(float x);
-    void setBRY(float y);
+    void signalDevListUpdate(const QList<KSaneCore::DeviceInfo> &deviceList);
+    void imageReady(const QImage &image);
+    
+private Q_SLOTS:
+
+    void setTLX(const QVariant &x);
+    void setTLY(const QVariant &y);
+    void setBRX(const QVariant &x);
+    void setBRY(const QVariant &y);
 
     void checkInvert();
     void invertPreview();
-    void pollPollOptions();
 
     void updateScanSelection();
     void setPossibleScanSizes();
     void setPageSize(int index);
+    
+    void updateCommonGamma();
+    void updatePreviewViewer();
+    
+public:
+    void alertUser(KSaneCore::KSaneScanStatus status, const QString &strStatus);
 
 public:
-    void alertUser(int type, const QString &strStatus);
-
-public:
+    KSaneCore          *m_ksaneCoreInterface;
+    
     // backend independent
     QTabWidget         *m_optsTabWidget;
     QScrollArea        *m_basicScrollA;
@@ -116,7 +105,6 @@ public:
     QWidget            *m_colorOpts;
     QScrollArea        *m_otherScrollA;
     QWidget            *m_otherOptsTab;
-    LabeledCheckbox    *m_invertColors;
 
     QVector<int>        m_sizeCodes;
     LabeledCombo       *m_scanareaPapersize;
@@ -145,15 +133,8 @@ public:
     QProgressBar       *m_progressBar;
     QPushButton        *m_cancelBtn;
 
-    // device info
-    SANE_Handle         m_saneHandle;
-    QString             m_devName;
-    QString             m_vendor;
-    QString             m_model;
-
     // Option variables
-    QList<KSaneOption *> m_optList;
-    QList<KSaneOption *> m_pollList;
+    QSet<QString>            m_handledOptions;
     KSaneOption        *m_optSource;
     KSaneOption        *m_optNegative;
     KSaneOption        *m_optFilmType;
@@ -170,6 +151,7 @@ public:
     KSaneOption        *m_optGamR;
     KSaneOption        *m_optGamG;
     KSaneOption        *m_optGamB;
+    KSaneOption        *m_optInvert;
     LabeledCheckbox    *m_splitGamChB;
     LabeledGamma       *m_commonGamma;
     KSaneOption        *m_optWaitForBtn;
@@ -181,28 +163,11 @@ public:
     QImage              m_previewImg;
     bool                m_isPreview;
     bool                m_autoSelect;
-
+    
+    bool                m_cancelMultiScan = false;
+    bool                m_scanOngoing = false;
     int                 m_selIndex;
 
-    bool                m_scanOngoing;
-    bool                m_closeDevicePending;
-    bool                m_cancelMultiScan = false;
-
-    // final image data
-    QByteArray          m_scanData;
-
-    // option handling
-    QTimer              m_readValsTmr;
-    QTimer              m_updProgressTmr;
-    QTimer              m_optionPollTmr;
-    KSaneScanThread    *m_scanThread;
-    KSanePreviewThread *m_previewThread;
-
-    QString             m_saneUserName;
-    QString             m_sanePassword;
-
-    FindSaneDevicesThread *m_findDevThread;
-    KSaneAuth             *m_auth;
     KSaneWidget           *q;
 };
 
